/*
 *    Copyright  1996, 1997, 1998, 1999, 2000 by Remco Treffkorn
 *    Distributed under the terms of the Q Public License. Please see
 *    the included file LICENSE.
 */

#include "config.h"
#include <stdio.h>

#include <Xm/DialogS.h>
#include <Xm/ScrolledW.h>
#include <Xm/DrawingA.h>
#include <Xm/MwmUtil.h>
#include "math.h"
#include "xmap.h"
#include "values.h"
#include "debug.h"

struct {
    Pixmap pm;
    double max;
} speeda;

extern XtAppContext app;
extern Widget toplevel;

extern Pixmap speedo100, speedo200, speedo999;

static GC speedGC;
static GC aspeedGC;
static GC asdelGC;
static GC positionGC;
static GC utmposGC;
static Widget speedw;
static Widget sdraww;
static Widget analogsw;
static Widget sadraww;
static Widget pdraww;
static Widget updraww;
static Widget positionw;
static Widget utmposw;

static void update_position();
static void update_digital_speed();
static void update_analog_speed();

#define TEST 0

#if TEST
static void draw_meter(Widget w, GC gc, int min, int max, int devisions);
#endif

static char *altitude, *velocity;
XFontStruct *big_font, *small_font;
XmFontList  big_fl;

static int alastx;
static int alasty;

void instrument_init(void)
{
    big_font = XLoadQueryFont(XtDisplay(toplevel),
		  "-*-lucida-medium-r-normal-*-*-240-*-*-*-*-iso8859-*");
    small_font = XLoadQueryFont(XtDisplay(toplevel), "fixed");

    big_fl = XmFontListCreate(big_font, "BigFont");
}


static void sredraw_cb()
{
    update_digital_speed();
}

static void do_speedometer()
{
    Arg args[100];
    XmString s;
    int n;
    Position x, y;
    Dimension w, h;
    char bufp[256];

    if (!speedw) {
	XGCValues gcv;

	speedw = XtVaCreatePopupShell("speedo",
				      xmDialogShellWidgetClass, toplevel,
				      XmNtitle, "",
				      XmNminWidth, 120,
				      XmNmaxWidth, 120,
				      XmNminHeight, 40,
				      XmNmaxHeight, 40,

				      XmNmwmFunctions, 0,
				      XmNmwmDecorations, MWM_DECOR_BORDER,
				      NULL);

	XtVaGetValues(toplevel,
		      XmNx, &x,
		      XmNy, &y,
		      XmNwidth, &w,
		      XmNheight, &h,
		      NULL);

	if (w > 100 && h > 40) {
	    y = (h - 40) / 2 + y;
	    x = (w - 100) / 2 + x;
	} else {
	    x += 10;
	    y += 10;
	}
	snprintf(bufp, 255, "+%d+%d", x, y);

	GetPrivateProfileString("Layout", "digispeed", bufp, bufp, 255, 
				inifile);

	XtVaSetValues(speedw, XmNgeometry, bufp, NULL);

	sdraww = XtVaCreateWidget("sdraw",
				  xmDrawingAreaWidgetClass, speedw,
				  XmNwidth, 120,
				  XmNheight, 40,
				  NULL);

	gcv.foreground = BlackPixelOfScreen(XtScreen(sdraww));
	speedGC = XCreateGC(XtDisplay(sdraww),
	       RootWindowOfScreen(XtScreen(sdraww)), GCForeground, &gcv);


	XtAddCallback(sdraww, XmNexposeCallback, sredraw_cb, NULL);
    }
    XtManageChild(sdraww);
    XtManageChild(speedw);
    XtPopup(speedw, XtGrabNone);
}

static void analog_speed_cb()
{
    Position width, height;

    if (analogsw) {
	XtVaGetValues(sadraww, XmNwidth, &width, XmNheight, &height, NULL);
	XCopyArea(XtDisplay(sadraww), speeda.pm, XtWindow(sadraww), aspeedGC,
		  0, 0, width, height, 0, 0);

	alastx = 0;
	alasty = 0;

#if TEST
	draw_meter(sadraww, aspeedGC, 0, 150, 15);
#endif
	update_analog_speed();
    }
}

static void do_analog_speed()
{
    Arg args[100];
    XmString s;
    int n;
    Position x, y;
    Dimension w, h;
    char bufp[256];

    if (!analogsw) {
	XGCValues gcv;

//tf. get values from .ini file

	speeda.max = (double) GetPrivateProfileInt("General", "analog_max", 100, inifile);

	switch ((int)speeda.max) {
	    default:
		speeda.max = 100.0;
	    case 100:
		speeda.pm  = speedo100;
		break;

	    case 200:
		speeda.pm  = speedo200;
		break;

	    case 1000:
		speeda.pm  = speedo999;
		break;
	}
//tf.

	analogsw = XtVaCreatePopupShell("speeda",
				      xmDialogShellWidgetClass, toplevel,
					XmNtitle, "",
					XmNminWidth, 162,
					XmNmaxWidth, 162,
					XmNminHeight, 162,
					XmNmaxHeight, 162,

					XmNmwmFunctions, 0,
				     XmNmwmDecorations, MWM_DECOR_BORDER,
					NULL);

	XtVaGetValues(toplevel,
		      XmNx, &x,
		      XmNy, &y,
		      XmNwidth, &w,
		      XmNheight, &h,
		      NULL);

	if (w > 293 && h > 239) {
	    y = (h - 239) / 2 + y;
	    x = (w - 293) / 2 + x;
	} else {
	    x += 100;
	    y += 100;
	}
	snprintf(bufp, 255, "+%d+%d", x, y);

	GetPrivateProfileString("Layout", "speedometer", bufp, bufp, 255, 
				inifile);

	XtVaSetValues(analogsw, XmNgeometry, bufp, NULL);

	sadraww = XtVaCreateWidget("draw",
				   xmDrawingAreaWidgetClass, analogsw,
				   XmNwidth, 162,
				   XmNheight, 162,
				   NULL);

	gcv.foreground = BlackPixelOfScreen(XtScreen(sadraww));
	gcv.line_width = 5;
	gcv.cap_style = CapProjecting;
	aspeedGC = XCreateGC(XtDisplay(sadraww),
			     RootWindowOfScreen(XtScreen(sadraww)),
			  GCForeground | GCLineWidth | GCCapStyle, &gcv);



	gcv.function = GXcopy;
	gcv.tile = speeda.pm;
	gcv.fill_style = FillTiled;
	gcv.line_width = 5;
	gcv.cap_style = CapProjecting;
	asdelGC = XCreateGC(XtDisplay(sadraww),
			    RootWindowOfScreen(XtScreen(sadraww)),
			    GCFillStyle | GCFunction | GCTile | GCLineWidth | GCCapStyle, &gcv);



	XtAddCallback(sadraww, XmNexposeCallback, analog_speed_cb, NULL);
    }
    XtManageChild(sadraww);
    XtManageChild(analogsw);
    XtPopup(analogsw, XtGrabNone);
}


static void predraw_cb()
{
    update_position();
}

static void do_position()
{
    Arg args[100];
    XmString s;
    int n;
    Position x, y;
    Dimension w, h;
    char bufp[256];
    
    if (!positionw) {
	XGCValues gcv;

	positionw = XtVaCreatePopupShell("position",
				      xmDialogShellWidgetClass, toplevel,
					 XmNtitle, "",
					 XmNminWidth, 240,
					 XmNmaxWidth, 240,
					 XmNminHeight, 80,
					 XmNmaxHeight, 80,

					 XmNmwmFunctions, 0,
				     XmNmwmDecorations, MWM_DECOR_BORDER,
					 NULL);

	XtVaGetValues(toplevel,
		      XmNx, &x,
		      XmNy, &y,
		      XmNwidth, &w,
		      XmNheight, &h,
		      NULL);

	if (w > 100 && h > 40) {
	    y = (h - 40) / 2 + y;
	    x = (w - 100) / 2 + x;
	} else {
	    x += 10;
	    y += 10;
	}
	snprintf(bufp, 255, "+%d+%d", x, y);

	GetPrivateProfileString("Layout", "digiposition", bufp, bufp, 255, 
				inifile);

	XtVaSetValues(positionw, XmNgeometry, bufp, NULL);

	pdraww = XtVaCreateWidget("pdraw",
				  xmDrawingAreaWidgetClass, positionw,
				  XmNwidth, 240,
				  XmNheight, 80,
				  NULL);

	gcv.foreground = BlackPixelOfScreen(XtScreen(pdraww));
	positionGC = XCreateGC(XtDisplay(pdraww),
	       RootWindowOfScreen(XtScreen(pdraww)), GCForeground, &gcv);


	XtAddCallback(pdraww, XmNexposeCallback, predraw_cb, NULL);
    }
    XtManageChild(pdraww);
    XtManageChild(positionw);
    XtPopup(positionw, XtGrabNone);
}


static void do_utmpos()
{
    Arg args[100];
    XmString s;
    int n;
    Position x, y;
    Dimension w, h;
    char bufp[256];

    if (!utmposw) {
	XGCValues gcv;

	utmposw = XtVaCreatePopupShell("utmposition",
				       xmDialogShellWidgetClass, toplevel,
				       XmNtitle, "",
				       XmNminWidth, 240,
				       XmNmaxWidth, 240,
				       XmNminHeight, 80,
				       XmNmaxHeight, 80,

				       XmNmwmFunctions, 0,
				     XmNmwmDecorations, MWM_DECOR_BORDER,
				       NULL);

	XtVaGetValues(toplevel,
		      XmNx, &x,
		      XmNy, &y,
		      XmNwidth, &w,
		      XmNheight, &h,
		      NULL);

	if (w > 100 && h > 40) {
	    y = (h - 40) / 2 + y;
	    x = (w - 100) / 2 + x;
	} else {
	    x += 10;
	    y += 10;
	}
	snprintf(bufp, 255, "+%d+%d", x, y);

	GetPrivateProfileString("Layout", "utmdigipos", bufp, bufp, 255, 
				inifile);


	XtVaSetValues(utmposw, XmNgeometry, bufp, NULL);

	updraww = XtVaCreateWidget("updraw",
				   xmDrawingAreaWidgetClass, utmposw,
				   XmNwidth, 240,
				   XmNheight, 80,
				   NULL);

	gcv.foreground = BlackPixelOfScreen(XtScreen(updraww));
	utmposGC = XCreateGC(XtDisplay(updraww),
	      RootWindowOfScreen(XtScreen(updraww)), GCForeground, &gcv);


	XtAddCallback(updraww, XmNexposeCallback, predraw_cb, NULL);
    }
    XtManageChild(updraww);
    XtManageChild(utmposw);
    XtPopup(utmposw, XtGrabNone);
}

handle_instrument_visibility()
{
    if (options.instruments == TRUE) {
	if (options.speedometer == TRUE)
	    do_speedometer();
	else if (speedw)
	    XtPopdown(speedw);

	if (options.aspeedometer == TRUE)
	    do_analog_speed();
	else if (analogsw)
	    XtPopdown(analogsw);

	if (options.position == TRUE)
	    do_position();
	else if (positionw)
	    XtPopdown(positionw);

	if (options.utmpos == TRUE)
	    do_utmpos();
	else if (utmposw)
	    XtPopdown(utmposw);

    } else {
	if (speedw)
	    XtPopdown(speedw);
	if (analogsw)
	    XtPopdown(analogsw);
	if (positionw)
	    XtPopdown(positionw);
	if (utmposw)
	    XtPopdown(utmposw);
    }
}

void instrument_cb(Widget w, XtPointer client_data, XmToggleButtonCallbackStruct * state)
{
    options.instruments = state->set;
    handle_instrument_visibility();
}

void aspeedometer_cb(Widget w, XtPointer client_data, XmToggleButtonCallbackStruct * state)
{
    options.aspeedometer = state->set;
    handle_instrument_visibility();
}

void speedometer_cb(Widget w, XtPointer client_data, XmToggleButtonCallbackStruct * state)
{
    options.speedometer = state->set;
    handle_instrument_visibility();
}

void position_cb(Widget w, XtPointer client_data, XmToggleButtonCallbackStruct * state)
{
    options.position = state->set;
    handle_instrument_visibility();
}

void utmpos_cb(Widget w, XtPointer client_data, XmToggleButtonCallbackStruct * state)
{
    options.utmpos = state->set;
    handle_instrument_visibility();
}

static void update_position()
{
    char buffer[32];
    char zone[8];
    int width, x;
    double utmx, utmy;

    // we are only called in case of change or an expose.

    if (options.position) {
	XClearWindow(XtDisplay(pdraww), XtWindow(pdraww));

	set_font(positionGC, big_font);
	toUserFormatLat(options.format, cv.latitude, buffer, 32);
	x = 240 - 4 - XTextWidth(big_font, buffer, strlen(buffer));
	XDrawString(XtDisplay(pdraww), XtWindow(pdraww), positionGC,
		x, 35, buffer, strlen(buffer));
	toUserFormatLon(options.format, cv.longitude, buffer, 32);
	x = 240 - 4 - XTextWidth(big_font, buffer, strlen(buffer));
	XDrawString(XtDisplay(pdraww), XtWindow(pdraww), positionGC,
		x, 75, buffer, strlen(buffer));
    }
    if (options.utmpos) {
	XClearWindow(XtDisplay(updraww), XtWindow(updraww));

	DegToUTM(cv.latitude, cv.longitude, zone, &utmx, &utmy);
	set_font(utmposGC, big_font);
	snprintf(buffer,31,"%.0lfE",utmx);
	x = 240 - 4 - XTextWidth(big_font, buffer, strlen(buffer));
	XDrawString(XtDisplay(updraww), XtWindow(updraww), utmposGC, x, 35, 
		    buffer, strlen(buffer));

	snprintf(buffer,31,"%.0lfN",utmy);
	x = 240 - 4 - XTextWidth(big_font, buffer, strlen(buffer));
	XDrawString(XtDisplay(updraww), XtWindow(updraww), utmposGC, x, 75, 
		    buffer, strlen(buffer));
    }
}


static void update_digital_speed()
{
    char buffer[32];
    int width, x;

    // we are only called in case of change or an expose.

    snprintf(buffer, 31, "%.1lf", cv.velocity_val);

    XClearWindow(XtDisplay(sdraww), XtWindow(sdraww));

    if (cv.velocity) {
	width = XTextWidth(small_font, cv.velocity, strlen(cv.velocity));
	set_font(speedGC, small_font);
	XDrawString(XtDisplay(sdraww), XtWindow(sdraww), speedGC, 118 - width,
		    35, cv.velocity, strlen(cv.velocity));
    }
    x = 120 - 4 - width - XTextWidth(big_font, buffer, strlen(buffer));
    set_font(speedGC, big_font);
    XDrawString(XtDisplay(sdraww), XtWindow(sdraww), speedGC, x, 35, buffer, 
		strlen(buffer));
}

#define XCENTER         (double)(162/2)
#define YCENTER         (double)(162/2)
#define SCALE           (double)(150/2)

static void update_analog_speed()
{
    char buffer[32];
    int width, x;
    double velocity;
    int xout, yout;

    // we are only called in case of change or an expose.

    if (cv.velocity_val > speeda.max /*100.0*/)
	velocity = speeda.max /*100.0*/;
    else
	velocity = cv.velocity_val;

//    velocity *= 3;		// makes it max 300 (degrees), that is what we have.
    velocity *= 300.0/speeda.max;    // makes it max 300 (degrees), that is what we have.

    velocity += 210;		// this is where 0 is. (0 degrees ist at 12 o'clock)

    velocity *= DEGREE;
    xout = XCENTER + sin(velocity) * SCALE;
    yout = YCENTER - cos(velocity) * SCALE;

    if (cv.velocity) {
	width = XTextWidth(small_font, cv.velocity, strlen(cv.velocity));
	set_font(aspeedGC, small_font);
	set_color(aspeedGC, "white");
	XDrawString(XtDisplay(sadraww), XtWindow(sadraww), aspeedGC, (161 - width) / 2, 145,
		    cv.velocity, strlen(cv.velocity));
    }
    if (alastx && alasty)
	XDrawLine(XtDisplay(sadraww), XtWindow(sadraww), asdelGC, 81, 81, alastx, alasty);
    set_color(aspeedGC, "red");
    XDrawLine(XtDisplay(sadraww), XtWindow(sadraww), aspeedGC, 81, 81, xout, yout);
    alastx = xout;
    alasty = yout;
}

void update_instruments()
{
    if (options.instruments) {
	if (options.speedometer && (cv.update & (U_UNITS | U_VELOCITY)))
	    update_digital_speed();
	if (options.aspeedometer && (cv.update & (U_UNITS | U_VELOCITY)))
	    update_analog_speed();
	if (options.position && (cv.update & (U_UNITS | U_VELOCITY)))
	    update_position();
    }
}

void save_instrument_properties()
{
    save_position(speedw, "digispeed");
    save_position(analogsw, "speedometer");
    save_position(positionw, "digiposition");
    save_position(utmposw, "utmdigipos");
}


#if TEST
static void draw_meter(Widget w, GC gc, int min, int max, int devisions)
{
    int start = 30;
    int end = 330;
    int span = 300;
    XGCValues gcv;
    double angle;
    int xstart, xend;
    int ystart, yend;
    int increment;
    int i;


    increment = span / devisions;

    set_color(gc, "yellow");

    for (i = 0; i < devisions; i++) {
	angle = 210.0 + (double) (increment * i);
	angle *= DEGREE;
	xstart = XCENTER + sin(angle) * (SCALE - 10);
	ystart = YCENTER - cos(angle) * (SCALE - 10);
	xend = XCENTER + sin(angle) * (SCALE);
	yend = YCENTER - cos(angle) * (SCALE);
	XDrawLine(XtDisplay(w), XtWindow(w), gc, xstart, ystart, xend, yend);
    }
}

#endif
