/*
 *    Copyright  1996, 1997, 1998, 1999, 2000 by Remco Treffkorn
 *    Distributed under the terms of the Q Public License. Please see
 *    the included file LICENSE.
 */
#include <stdio.h>

#include <Xm/DialogS.h>
#include <Xm/ScrolledW.h>
#include <Xm/DrawingA.h>
#include <Xm/MwmUtil.h>
#include "xmap.h"
#include "debug.h"

// Here you have it: 

extern XtAppContext app;
extern Widget toplevel;

static Widget dialog = NULL;
static Widget draww;


static GC gc;
extern Pixmap splash;
static Dimension width, height;
static XtIntervalId id;


static void input_cb(Widget w, XtPointer data, XmDrawingAreaCallbackStruct * cbs)
{
    XEvent *e = cbs->event;

    if (e->xany.type == ButtonRelease) {
	if (dialog)
	    XtDestroyWidget(dialog);
	dialog = NULL;
    }
}

static void to_cb(Widget w, XtPointer data, XmDrawingAreaCallbackStruct * cbs)
{
    if (dialog)
	XtDestroyWidget(dialog);
    dialog = NULL;
}

static void redraw_cb()
{
    if (splash) {
	char bufp[256];

	snprintf(bufp, 255, "Hugo");
	XDrawString(XtDisplay(draww), splash, gc,
		    140, 30, bufp, strlen(bufp));

	snprintf(bufp, 255, "Version %s", VERSION);
	XDrawString(XtDisplay(draww), splash, gc,
		    110, 120, bufp, strlen(bufp));


	snprintf(bufp, 255, "Build %s %s", __DATE__, __TIME__);
	XDrawString(XtDisplay(draww), splash, gc,
		    80, 150, bufp, strlen(bufp));

	XtVaGetValues(draww, XmNwidth, &width, XmNheight, &height, NULL);
	XCopyArea(XtDisplay(draww), splash, XtWindow(draww), gc,
		  0, 0, width, height, 0, 0);

    }
}

Widget findTopLevelShellParent(Widget w)
{
    while (w != 0 && (!XtIsWidget(w)
		      || !XtIsTopLevelShell(w)
		      || !XtIsRealized(w)
		      || XtDisplay(w) == 0
		      || XtScreen(w) == 0
		      || XtWindow(w) == 0))
	w = XtParent(w);
    return w;
}


Widget find_shell(Widget w)
{
    XWindowAttributes xwa;

    Widget parent = findTopLevelShellParent(w);
    if (parent == 0 || !XtIsRealized(parent))
	return toplevel;

    XGetWindowAttributes(XtDisplay(parent), XtWindow(parent), &xwa);
    if (xwa.map_state != IsViewable)
	return toplevel;

    return parent;
}


void wait_until_mapped(Widget w, Widget shell)
{
    XEvent event;
    XWindowAttributes attr;

    XSync(XtDisplay(w), False);
    XmUpdateDisplay(w);

    if (shell == 0)
	shell = find_shell(w);

    if (XtIsRealized(w) && XtIsRealized(shell)) {
	while (XGetWindowAttributes(XtDisplay(w), XtWindow(w), &attr)
	       && attr.map_state != IsViewable) {

	    if (XGetWindowAttributes(XtDisplay(shell), XtWindow(shell), &attr)
		&& attr.map_state != IsViewable)
		break;

	    XtAppNextEvent(XtWidgetToApplicationContext(w), &event);
	    XtDispatchEvent(&event);
	}
    }
    XSync(XtDisplay(w), False);
    XmUpdateDisplay(w);
}

static void really_do_splash(int f)
{
    Arg args[100];
    XmString s;
    int n;
    Position x, y;
    Dimension w, h;
    char bufp[256];

    if (!dialog) {
	XGCValues gcv;

	dialog = XtVaCreatePopupShell("splash",
				      xmDialogShellWidgetClass, toplevel,
			   XmNtitle, "Where ever you go, there you are!",
				      XmNminWidth, 293,
				      XmNmaxWidth, 293,
				      XmNminHeight, 239,
				      XmNmaxHeight, 239,

				      XmNmwmFunctions, 0,
				      XmNmwmDecorations, MWM_DECOR_BORDER,
				      NULL);

	XtVaGetValues(toplevel,
		      XmNx, &x,
		      XmNy, &y,
		      XmNwidth, &w,
		      XmNheight, &h,
		      NULL);

	if (w > 293 && h > 239) {
	    y = (h - 239) / 2 + y;
	    x = (w - 293) / 2 + x;
	} else {
	    x += 100;
	    y += 100;
	}
	snprintf(bufp, 255, "%dx%d+%d+%d", 293, 239, x, y);

	XtVaSetValues(dialog, XmNgeometry, bufp, NULL);

	draww = XtVaCreateWidget("draw",
				 xmDrawingAreaWidgetClass, dialog,
				 XmNwidth, 293,
				 XmNheight, 239,
				 NULL);

	gcv.foreground = BlackPixelOfScreen(XtScreen(draww));
	gc = XCreateGC(XtDisplay(draww),
		RootWindowOfScreen(XtScreen(draww)), GCForeground, &gcv);


	XtAddCallback(draww, XmNexposeCallback, (XtCallbackProc)redraw_cb, NULL);
	XtAddCallback(draww, XmNinputCallback, (XtCallbackProc)input_cb, NULL);

    }
    XtManageChild(draww);
    XtManageChild(dialog);
    XtPopup(dialog, XtGrabNone);
    if (f) {
	wait_until_mapped(draww, dialog);
	XtAppAddTimeOut(app, 3000L, (XtTimerCallbackProc) to_cb, toplevel);
    }
}

void do_splash()
{
    really_do_splash(1);
    return;
}

void do_about()
{
    really_do_splash(0);
    return;
}
