/*
 *    Copyright  1996, 1997, 1998, 1999, 2000 by Remco Treffkorn
 *    Distributed under the terms of the Q Public License. Please see
 *    the included file LICENSE.
 */
/******************************************************************************
 *  legend.c   draw the scale/legend box                                      *
 ******************************************************************************/
#include "vector.h"
#include "units.h"
#include <stdio.h>
#include <math.h>

void DrawLegend(ms)
    MapStruct   *ms;
  {
    Pixmap  image;
    char        text[80];
    int         i,y=25,x=6;
#if 0
    map_params(ms);
    if (ms->theme.on) ReadConfig(ms,"thematic.con");
	  else ReadConfig(ms,"mapnew.con");
    image = gdImageCreate(350,150);
    gdImageColorAllocate(image,255,255,255);
    gdImageColorAllocate(image,0,0,0);
    /* Allocate Colors */
    for (i=0; i<ms->ndirs; i++) {
        if ((ms->datasets[i].on>0) && (strlen(ms->datasets[i].label)>0)) {
            AllocColor(image,&ms->datasets[i].line);
            AllocColor(image,&ms->datasets[i].fill);
            AllocColor(image,&ms->datasets[i].point);
            DrawLegendItem(image,&ms->datasets[i],ms->scale,&x,&y);
          }
      }
    for (i=0; i<ms->nlyrs; i++) {
        if ((ms->layers[i].on>0) && (strlen(ms->layers[i].label)>0)) {
			if (ms->theme.on && !strcmp(ms->layers[i].name,ms->theme.level))
				DrawThematicLegend(image,ms,i,&x,&y);
	          else {
                AllocColor(image,&ms->layers[i].line);
                AllocColor(image,&ms->layers[i].fill);
                AllocColor(image,&ms->layers[i].point);
                DrawLegendItem(image,&ms->layers[i],ms->scale, &x,&y);
	          }
          }
      }
    for (i=0; i<ms->nfeats; i++) {
        if ((ms->features[i].on>0) && (strlen(ms->features[i].label)>0) ) {
            AllocColor(image,&ms->features[i].line);
            AllocColor(image,&ms->features[i].fill);
            AllocColor(image,&ms->features[i].point);
            DrawLegendItem(image,&ms->features[i],ms->scale, &x,&y);
          }
      }
/* write title */
    gdImageString(image, gdFontLarge, 151,8,"LEGEND", BLACK);

    /* Scale & scale bar */
    sprintf(text,"Scale 1:%i",ms->scale);
    gdImageString(image,gdFontLarge, 5,127,text,BLACK);
    strcpy(text,"*average--true scale depends on monitor resolution");
    gdImageString(image,gdFontSmall, 5,138,text,BLACK);

    DrawScaleBar(ms,image);
    
    /* Make image Interlaced */
    gdImageInterlace(image,1);
    gdImageGif(image, stdout);
    gdImageDestroy(image);
  }

void DrawScaleBar(MapStruct *ms, gdImagePtr image)
  {
    float mu, miles, feet, meters, km, mantissa, expn, lg,inc,len,uperpix,y; 
    int   incs[9],lens[9],i,x;
    char  unit[3],text[10];
    
    /* initialize length and increment settings */
    lens[0]=10;incs[0]=2;  lens[1]=15;incs[1]=5;  lens[2]=20;incs[2]=5;
    lens[3]=25;incs[3]=5;  lens[4]=30;incs[4]=5;  lens[5]=40;incs[5]=10;
    lens[6]=50;incs[6]=10;  lens[7]=60;incs[7]=10;  lens[8]=80;incs[8]=20;
    
    /* figure out lengths which will fit on bar */
    uperpix=ms->muPerPix*MI_PER_DEG/LU_PER_DEG; /* miles per pixel */
    miles = 180*uperpix; /* # of miles available */
    feet = miles * 5280;
    meters = feet * .3048;
    km = meters /1000;
    /* setup english scale parameters */
    if (miles >=1) {  /* show scale in miles */
        lg = log10(miles);
        strcpy(unit,"mi");
      } else { /* show scale in feet */
        lg = log10(feet);
        strcpy(unit,"ft");
        uperpix = uperpix * 5280;
      }
    mantissa = pow(10,lg-(int)lg+1);
    expn = pow(10,((int)lg)-1);
    for (i=8; i>=0; i--) {
        if (lens[i]<mantissa) {
            len=lens[i]*expn;
            inc=incs[i]*expn;
            i=-1;
          }
      }
    /* Plot english scale */
    for (y=0; y<=len; y+=inc) {
        x=(int)(135+y/uperpix);
        gdImageLine(image, x,129,x,123, BLACK);
        if (inc<1) sprintf(text,"%.1f",y);
          else sprintf(text,"%.0f",y);
        gdImageString(image,gdFontSmall, x+3,119,text,BLACK);
      }
    gdImageLine(image, 135, 129, x,129, BLACK);
    x+=15+6*((int)lg+(inc<1));
    gdImageString(image,gdFontSmall,x,119,unit,BLACK);

    /* setup metric scale parameters */
    if (km >=1) {  /* show scale in kilometers */
        lg = log10(km);
        strcpy(unit,"km");
        uperpix=ms->muPerPix*MI_PER_DEG*5.28/(LU_PER_DEG*FT_PER_M);
      } else { /* show scale in meters */
        lg = log10(meters);
        strcpy(unit,"m");
        uperpix=ms->muPerPix*MI_PER_DEG*5280/(LU_PER_DEG*FT_PER_M);
      }
    mantissa = pow(10,lg-(int)lg+1);
    expn = pow(10,(int)lg-1);
    for (i=8; i>=0; i--) {
        if (lens[i]<mantissa) {
            len=lens[i]*expn;
            inc=incs[i]*expn;
            i=-1;
          }
      }
    /* plot metric scale */
    for (y=0; y<=len; y+=inc) {
        x=(int)(135+y/uperpix);
        gdImageLine(image, x,130,x,136, BLACK);
        if (inc<1) sprintf(text,"%.1f",y);
          else sprintf(text,"%.0f",y); 
        gdImageString(image,gdFontSmall, x+3,130,text,BLACK);
      }
    gdImageLine(image, 135, 130, x,130, BLACK);
    x+=15+6*((int)lg+(inc<1));
    gdImageString(image,gdFontSmall,x,130,unit,BLACK);
  }

void DrawLegendItem(gdImagePtr image,Config *config, long scale, int *x2,int *y2)
  {
     Symbol  *linesym,*fillsym,*ptsym;
     int y=*y2,x=*x2;

    if (config->line.color != -1) linesym = &config->line;
	  else linesym = NULL;
    if (config->fill.color != -1) fillsym = &config->fill;
	  else fillsym = NULL;
    if (config->point.color != -1) ptsym = &config->point;
	  else ptsym = NULL;
     if ((fillsym == NULL) && (linesym == NULL) && (ptsym == NULL)) {
         return;
       }
     if (fillsym != NULL) {
         if (fillsym->brush == NULL)
            gdImageFilledRectangle(image,x,y,x+23,y+10,fillsym->color);
          else {
            gdImageSetBrush(image, fillsym->brush);
            gdImageFilledRectangle(image,x,y,x+23,y+10,gdBrushed);
          }
         if (linesym != NULL) { 
             if (linesym->brush == NULL)
                gdImageRectangle(image,x,y,x+23,y+10,linesym->color);
              else {
                gdImageSetBrush(image, linesym->brush);
                gdImageRectangle(image,x,y,x+23,y+10,gdBrushed);
              }
           }
       } else if (linesym != NULL) {
         if (linesym->brush == NULL)
            gdImageLine(image,x,y+5,x+23,y+5,linesym->color);
          else {
            gdImageSetBrush(image, linesym->brush);
            gdImageLine(image,x,y+5,x+23,y+5,gdBrushed);
          }
       }
     if (ptsym != NULL) {
         if (ptsym->brush == NULL)
            gdImageLine(image,x+20,y+5,x+20,y+5,ptsym->color);
          else {
            gdImageSetBrush(image, ptsym->brush);
            gdImageLine(image,x+20,y+5,x+20,y+5,gdBrushed);
          }
      }

     gdImageString(image,gdFontLarge,x+30,y,config->label,BLACK);

     *y2+=14;
     if (*y2>110) {
         *x2+=175;
         *y2=25;
       }
  }

/* Draw legend items for Thematic map */
void DrawThematicLegend(gdImagePtr image,MapStruct *ms, int layer,int *x2, int *y2)
  {
	char	msg[100];
	char 	var[20],meth[2],vals[100],name[50],*stop,qfile[50],line[100];
	long	i,j,varnum;
	float	breaks[6];
    Symbol	linesym,fillsym[6];
	int	x=*x2,y=*y2;
	FILE	*dbfile;

    if (!strcmp(ms->theme.meth,"quantiles") || !strcmp(ms->theme.meth,"q")) {
	strcpy(meth,"q");
      } else if (!strcmp(ms->theme.meth,"intervals") || !strcmp(ms->theme.meth,"i")) {
      	strcpy(meth,"r");
      } else return;

    strtol(ms->theme.var,&stop,&varnum);
    /* read config file to get variable name and colors*/
    if (!(dbfile=fopen("/data/stats/fields","r"))) perror ("ThematicLegend: fopen fields");
    while (fscanf(dbfile,"%[^\n]\n",line) != EOF) {
        if (sscanf(line,"colors:%[^\n]\n",vals)) {
	    for (i=0;i<=5;i++) {
	        sscanf(vals,"%hi,%hi,%hi;%[^\n]\n",&fillsym[i].r,&fillsym[i].g,&fillsym[i].b,vals);
	        fillsym[i].file[0]=0;
	        AllocColor(image,&fillsym[i]);
	      }
	    sscanf(vals,"%hi,%hi,%hi",&linesym.r,&linesym.g,&linesym.b);
	    linesym.file[0]=0;
	    AllocColor(image,&linesym);
	  } else if (sscanf(line,"%i:%[^:]:%[^\n]\n",&j,vals,name)==3) {
	    if (!strcmp(vals,ms->theme.var) || (varnum==j)) {
		strcpy(var,name);
		varnum=j;
	      }
	  }
      }

    /* read class break values */
    strcpy(qfile,"/data/stats/");
    strcat(qfile,ms->theme.level);
    strcat(qfile,".");
    strcat(qfile,meth);
    if (!(dbfile=fopen(qfile,"r"))) perror ("ThematicLegend: fopen qfile");
    while (fscanf(dbfile,"%[^\n]\n",line) != EOF) {
	if (sscanf(line,"%i:%[^\n]\n",&j,vals) && (varnum==j)) {
           sscanf(vals,"%f,%f,%f,%f,%f,%f,%[^\n]\n",&breaks[0],&breaks[1],
                 &breaks[2],&breaks[3],&breaks[4],&breaks[5]);
	  }
      }

    /* Write Header Label */
    strcpy(vals,ms->layers[layer].label);
    gdImageString(image,gdFontLarge,x,y,vals,BLACK);
    y+=11;
    strcpy(vals,var);
    gdImageString(image,gdFontSmall,x+15,y,vals,BLACK);
    y+=10;
    /* Draw Class Boxes and Labels */
    for (i=0;i<6;i++) {
	gdImageFilledRectangle(image,x+15,y,x+40,y+8,fillsym[i].color);
	gdImageRectangle(image,x+15,y,x+40,y+8,linesym.color);
	if (i==0) 
	    strcpy(vals,"0.0");
	  else
	    sprintf(vals,"%.1f - %.1f",breaks[i-1]+.1,breaks[i]);
	gdImageString(image,gdFontSmall,x+45,y-1,vals,BLACK);
	y+=8;
      }
    y+=3;
    if (y>110) {
	*x2+=175;
	y=25;
      }
    *y2=y;
#endif
  }

